#!/usr/bin/env python

# -*- coding: utf-8 -*-
'''
Stores IPython notebooks automagically onto OpenStack clouds through Swift.
'''

__title__ = 'bookstore'
__version__ = '0.0.2a'
__build__ = 0x000002
__author__ = 'Kyle Kelley'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2013 Kyle Kelley'

import swift
import cloudfiles

